#!/usr/bin/python
# -*- coding: utf-8 -*-


import string
import re
import time


class SyslogLine(object):
  """Structural class for every line
  
  It happend to be a struct to accomodate a line data after exploding the string
  """
  def __init__(self,line_timestamp,src_host,src_process,process_data):
    self.timestamp=line_timestamp
    self.host=src_host
    self.process=src_process
    self.data=process_data
    
  def IsTheProcess(self,the_process):
    """Check for source process
    
    Used to discard lines that not belongs to a specific process
    """
    return string.find(self.process,the_process)>-1


class Syslog(object):
  """Base class for syslog input
  
  Contains attributes and methods that belongs to syslog, it isolates the "process data" for later processing
  """
  def __init__(self,log_year=2010):
    self._year=log_year
    self.syslog_format=re.compile('^(\w*\s\d*\s\d*:\d*:\d*)\s(\w*)\s([\w\[\]]*):(.*)')
    self.start_timestamp=False
    self.end_timestamp=False
    self.lines=[]
   
  def SourceFile(self,logfile):
    """The syslog file to use
    
    Just open the file for reading
    """
    self.file = open(logfile)
    
  def ParseLine(self,the_line):
    """Build a SyslogLine object for the_line
    
    It explodes the line and builds a SyslogLine object that returns
    """
    line_result=re.match(self.syslog_format,the_line)
    if line_result:
      line_timestamp=time.mktime(time.strptime('%d %s' % (self._year, line_result.group(1)),'%Y %b %d %H:%M:%S'))
      line_obj=SyslogLine(line_timestamp,line_result.group(2),line_result.group(3),string.strip(line_result.group(4)))
      return line_obj
    else:
      return False

  def PopulateLines(self,the_process=False):
    """Reads and parses all the lines in the file
    
    It uses the already opened file to read every line and parses it into SyslogLines storing it in self.lines
    """
    line=self.file.readline()
    last_line=None
    while not line == '':
      line_result=self.ParseLine(line)
      if line_result:
	if not self.start_timestamp:
	  self.start_timestamp=line_result.timestamp
	if the_process:
	  if line_result.IsTheProcess(the_process):
	    self.lines.append(line_result)
	else:
	  self.lines.append(line_result)
	last_line=line_result
      line=self.file.readline()
    self.file.close()
    self.end_timestamp=last_line.timestamp
    return self.lines


class Squid(Syslog):
  """Input class for line processing
  
  It's the base class for manipulating the data related to squid stored in the syslog file
  """
  def __init__(self):
    super(Squid,self).__init__()
    self.squid_format=re.compile('[\d\.]*\s*\d*\s(\d*\.\d*\.\d*\.\d*)\s(\w*/\d{3})\s(\d*)\s(GET|POST|CONNECT|HEAD)\s(.*)\s(\w*)\s\w*/\d*\.\d*\.\d*\.\d*.*')
    
  def ExplodeData(self):
    """Parses all SyslogLines in self.lines
    
    Used to process the squid data after the use of PopulateLines
    """
    last_index=len(self.lines)
    for line_index in range(last_index-1,-1,-1):
      self.lines[line_index]=self.ParseSquid(self.lines[line_index])
      if not self.lines[line_index]:
	del(self.lines[line_index])
    return self.lines
    
  def ParseSquid(self,the_line):
    """Get squid related data from SyslogLine.data
    
    Using re module gets the traffic data from already built SyslogLines
    """
    result=re.match(self.squid_format,the_line.data)
    if result:
      the_line.sourceIp=result.group(1)
      the_line.squidEvent=result.group(2)
      the_line.size=result.group(3)
      the_line.method=result.group(4)
      the_line.url=result.group(5)
      the_line.user=result.group(6)
      del(the_line.data)
      return the_line
    else:
      return False
      
  def Lines(self):
    """'The' method to use
    
    This is the only method needed in the regular use, it calls the PopulateLines to parse de syslog file and later ExplodeData to get the squid info from that lines
    """
    self.PopulateLines('squid')
    self.ExplodeData()
    return self.lines



#!/usr/bin/python 
# -*- coding: utf-8 -*-

import re

class Timeline(object):
  """Structural class for related data
  
  Used for ...
  """
  def __init__(self,start_timestamp,end_timestamp,genre=False,descriptor=False,descriptorValue=False):
    self.begining=int(start_timestamp)
    self.end=int(end_timestamp)
    self.genre=genre
    self.descriptor=descriptor
    self.descriptorValue=descriptorValue
    self.time={}
    
  def AddValue(self,time_stamp,value):
    time_stamp=int(time_stamp)
    if time_stamp not in self.time.keys():
      self.time[time_stamp]=value
    else:
      self.time[time_stamp]+=value
    return self.time[time_stamp]


def GetUsers(lines):
  """
  """
  start_timestamp=lines[0].timestamp
  end_timestamp=lines[len(lines)-1].timestamp
  users={}
  for line in lines:
    if line.user not in users.keys():
      users[line.user]=Timeline(start_timestamp,end_timestamp,'SquidUser','User',line.user)
    users[line.user].AddValue(line.timestamp,int(line.size))
  return users

def GetDomains(lines):
  """
  """
  start_timestamp=lines[0].timestamp
  end_timestamp=lines[len(lines)-1].timestamp
  domains={}
  domain_re=re.compile(r'(?:\w{3,5}://)?(\S*?)((:((443)|(5222))(?=$))|(/.*))')
  for line in lines:
    domain_result=re.match(domain_re,line.url)
    domain=domain_result.group(1)
    if domain not in domains.keys():
      domains[domain]=Timeline(start_timestamp,end_timestamp,'SquidDomain','Domain',domain)
    domains[domain].AddValue(line.timestamp,int(line.size))
  return domains

def GetNavigations(lines):
  """
  
  """
  start_timestamp=lines[0].timestamp
  end_timestamp=lines[len(lines)-1].timestamp
  navigations={}
  domain_re=re.compile(r'(?:\w{3,5}://)?(\S*?)((:((443)|(5222))(?=$))|(/.*))')
  for line in lines:
    domain_result=re.match(domain_re,line.url)
    if domain_result:
      if line.user+'@'+line.sourceIp+' >> '+domain_result.group(1) not in navigations.keys():
	navigations[line.user+'@'+line.sourceIp+' >> '+domain_result.group(1)]=Timeline(start_timestamp,end_timestamp,'SquidNavigation','Navigation',line.user+line.sourceIp+domain_result.group(1))
      navigations[line.user+'@'+line.sourceIp+' >> '+domain_result.group(1)].AddValue(line.timestamp,int(line.size))
    else:
      print line.url, line.squidEvent
  return navigations

def TopDomains(lines,cant=10):
  """
  """
  domains=GetDomains(lines)
  domains_sum=SumSizes(domains)
  domai=[]
  for domain in domains_sum.keys():
    domai.append((domains_sum[domain],domain))
  domai.sort()
  domai.reverse()
  domai=domai[0:cant-1]
  top_domains={}
  for trf_size, domain in domai:
    top_domains[domain]=domains[domain]
  return top_domains

def TopNavigations(lines,cant=10):
  """
  """
  navigations=GetNavigations(lines)
  navigations_sum=SumSizes(navigations)
  navi=[]
  for navigation in navigations_sum.keys():
    navi.append((navigations_sum[navigation],navigation))
  navi.sort()
  navi.reverse()
  navi=navi[0:cant-1]
  top_navigations={}
  for trf_size, navigation in navi:
    top_navigations[navigation]=navigations[navigation]
  return top_navigations


def EscapeNames(data,esc_char='_'):
  """
  """
  pattern=re.compile(r'(192.168.)|([^\w]+)')
  new_data={}
  for key in data.keys():
    new_data[re.sub(pattern,esc_char,key)[:19]]=data[key]
  return new_data

def CompleteData(data,complete_value=0):
  """
  """
  for time_stamp in range(data.begining,data.end+1):
    if time_stamp not in data.time:
      data.time[time_stamp]=complete_value
  return data

def SumSizes(data):
  """
  """
  sizes={}
  for desc in data.keys():
    sizes[desc]=0
    for time_stamp in data[desc].time.keys():
      sizes[desc]+=data[desc].time[time_stamp]
  return sizes

def SanitizeSizes(data):
  sizes = data.items()
  sizes.sort(key=lambda sizes:sizes[1], reverse=True)
  
  newsizes=[]
  for elem in sizes:
    newsizes.append((elem[0],__ConvertSizes(elem[1])))
  
  return newsizes

def __ConvertSizes(bytes):
    """Convert bytes to KB and  MB

    """
    if bytes >= 1048576:
      return str(round(bytes/1024.0/1024.0, 2)) + ' MB'
    elif bytes >= 1024:
      return str(round(bytes/1024.0, 2)) + ' KB'
    elif bytes < 1024:
      return str(bytes) + ' Bytes'




#!/usr/bin/python
# -*- coding: utf-8 -*-

import pylogan.input
import pylogan.process
import pylogan.output

#Need of a configuration file, this variables should be stored there
syslog_path='syslog'
out_path='webroot/'

page_title='Utilización del Proxy Squid'
page_users_block_title='Tráfico por Usuarios'
page_domains_block_title='Tráfico por Dominios'
page_navigations_block_title='Tráfico por Navegaciones'
page_traffic_heading='Tráfico'
page_user_heading='Usuario'
page_domain_heading='Dominio'
page_navigation_heading='Navegación'

#The first step is to read the file (just syslog fromat at the moment).
#There is just one implemented process, and is for squid because I could'nt find a log analyzer for squid capable of use syslog (there is modlogan but is cryptic and discontinued)
input_log=pylogan.input.Squid()
input_log.SourceFile(syslog_path)
lines=input_log.Lines()

#Fill the 'holes' in the timelines
users=pylogan.process.GetUsers(lines)
for user in users.keys():
  users[user]=pylogan.process.CompleteData(users[user])
users=pylogan.process.SumSizes(users)
users=pylogan.process.SanitizeSizes(users)
users.insert(0,(page_user_heading,page_traffic_heading))

domains=pylogan.process.TopDomains(lines)
for domain in domains.keys():
  domains[domain]=pylogan.process.CompleteData(domains[domain])
domains=pylogan.process.SumSizes(domains)
domains=pylogan.process.SanitizeSizes(domains)
domains.insert(0,(page_domain_heading,page_traffic_heading))

navigations=pylogan.process.TopNavigations(lines)
for navigation in navigations.keys():
  navigations[navigation]=pylogan.process.CompleteData(navigations[navigation])
navigations=pylogan.process.SumSizes(navigations)
navigations=pylogan.process.SanitizeSizes(navigations)
navigations.insert(0,(page_navigation_heading,page_traffic_heading))

#Build graphs
#users_graph=pylogan.output.RRDTool('users',out_path)
#users_graph.GraphTimelines(users,True)
#domains_graph=pylogan.output.RRDTool('domains',out_path)
#domains_graph.GraphTimelines(pylogan.process.EscapeNames(domains),True)
#navigations_graph=pylogan.output.RRDTool('navigations',out_path)
#navigations_graph.GraphTimelines(pylogan.process.EscapeNames(navigations),True)

#Build the XHTML page
page=pylogan.output.XHTML(out_path)
page.SetTitle(page_title)
page.CreateBlock('users',page_users_block_title)
#page.IncludeImage('users',users_graph.graphfile)
page.IncludeTable('users',users)
page.CreateBlock('domains',page_domains_block_title)
##page.IncludeImage('domains',domains_graph.graphfile)
page.IncludeTable('domains',domains)
page.CreateBlock('navigations',page_navigations_block_title)
##page.IncludeImage('navigations',navigations_graph.graphfile)
page.IncludeTable('navigations',navigations)
page.BuildContent()
page.WriteFile()

#End



#!/usr/bin/python
# -*- coding: utf-8 -*-

from distutils.core import setup

setup(name='pylogan',
      version='0.2.1',
      description='Modular Log Analayzer',
      author='Irving Leonard',
      author_email='irvingleonard@gmail.com',
      url='http://don.t.have.one',
      platforms=['POSIX'],
      license='BSD',
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'Operating System :: POSIX',
          'Programming Language :: Python',
          'Topic :: Communications',
          ],
      provides=['pylogan'],
      requires=['string','re','time','random','pyrrd','xml',],
      packages=['pylogan',],
      scripts=['pylogan_bin.py',],
      data_files=[
	  ('share/pylogan/webroot', ['webroot/pylogan.css',]),
	  ('share/pylogan/', ['README',]),
	  ],
     )